﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;

namespace Bewertung
{
    /// <summary>
    /// This class is the abstract variant for the chosen assessment approach.
    /// It holds all required and helpful dependencies between tasks, lemmata and their basics of assessment.
    /// </summary>
    public abstract class AbstractBewerter : IBewerter
    {
        /// <summary> Configuration of the Assessment </summary>
        protected int Round;
        /// <summary> Configuration of the Assessment </summary>
        protected Dictionary<int, double> weights;
        /// <summary> The objects of assessment - for each single task </summary>
        protected ImmutableList<AssessmentObject> bewertung;        

        /// <summary>
        /// initializes all required data for the assessment.
        /// </summary>
        /// <param name="con"></param>
        public AbstractBewerter(Container con)
        {
            Round = 5;
            weights = new Dictionary<int, double>()
            {
                [1] = 1.0,
                [2] = 0.25,
                [3] = 0.5,
                [4] = 0.25,
                [5] = 0.5,
                [6] = 0.25,
                [7] = 0.25
            };
            // Init Bewertung
            List<AssessmentObject> bewertung = new List<AssessmentObject>();
            Lemma mainProofLemma = new Lemma(con.Proof.proof.coords,"main proof", con.Task.task.eq, con.Task.task.fixedVars, con.Proof.proof);
            mainProofLemma.OwnTask = con.Task.task;
            mainProofLemma.usefulFor.Add(new TaskAssignment(con.Task.task));
            mainProofLemma.IsProved = true;
            List<Lemma> usefulLemmata = new List<Lemma>() { mainProofLemma };
            foreach (Lemma lemma in con.Proof.lemmata)
                if (lemma.GetTaskAssignment(con.Task.task) != null) usefulLemmata.Add(lemma);
            bewertung.Add(new AssessmentObject(con.Task.task, usefulLemmata));
            foreach (SingleTask task in con.Task.lemmaTasks)
            {
                mainProofLemma = null;
                usefulLemmata = new List<Lemma>();
                foreach (Lemma lemma in con.Proof.lemmata)
                    if (lemma.OwnTask == task) { mainProofLemma = lemma; break; }
                if (mainProofLemma != null)
                {
                    usefulLemmata.Add(mainProofLemma);
                    foreach (Lemma lemma in con.Proof.lemmata)
                        if (lemma.OwnTask == null && lemma.GetTaskAssignment(task) != null) usefulLemmata.Add(lemma);
                }
                bewertung.Add(new AssessmentObject(task, usefulLemmata));
            }
            this.bewertung = bewertung.ToImmutableList();
        }

        /// <summary>
        /// expands the 1 paramenter constructor by user defined configuration of the assessment.
        /// </summary>
        /// <param name="con"></param>
        /// <param name="config"></param>
        public AbstractBewerter(Container con, string config) : this(con)
        {
            string[] lines = config.Split('\n');
            foreach (string line in lines)
            {
                string[] words = line.Split(' ', 2);
                switch (words[0].ToLower())
                {
                    case "round": Round = Int32.Parse(words[1].Trim()); break;
                    case "1": weights[1] = Double.Parse(words[1].Trim()); break;
                    case "2": weights[2] = Double.Parse(words[1].Trim()); break;
                    case "3": weights[3] = Double.Parse(words[1].Trim()); break;
                    case "4": weights[4] = Double.Parse(words[1].Trim()); break;
                    case "5": weights[5] = Double.Parse(words[1].Trim()); break;
                    case "6": weights[6] = Double.Parse(words[1].Trim()); break;
                    case "7": weights[7] = Double.Parse(words[1].Trim()); break;
                    default: throw new Exception("Found unknown key " + words[0] + " in the config.");
                }
            }
        }

        /// <summary>
        /// (Re)initialises the assessment. Only the attribute "bewertung" is resetted, so every task has 0 points.
        /// </summary>
        protected void initializeBewertung()
        {
            for (int ID = 0; ID < bewertung.Count; ID++)
                bewertung[ID].init();
        }

        protected double confPt(int maxPt, double abstrMaxPt, int minSteps)
            => maxPt * Math.Min(1.0, abstrMaxPt / Math.Max(1.0,minSteps));

        public abstract void Bewerte();
        public ImmutableList<IAssessmentResult> GetBewertung()
        {
            List<IAssessmentResult> result = new List<IAssessmentResult>();
            foreach (AssessmentObject aO in bewertung)
            {
                List<InductionPartAssessment> ipaReadOnly = new List<InductionPartAssessment>();
                foreach (InductionPartAssessment ipa in aO.confIndAssessment)
                    ipaReadOnly.Add(new InductionPartAssessment(ipa));
                result.Add(new AssessmentResult(aO.task, aO.assessment, ipaReadOnly.ToImmutableList()));
            }
            return result.ToImmutableList();
        }
        public int GetRound() => Round;        

    }
}
